#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import wx
import numpy as np
import matplotlib as mpl
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg as NavigationToolbar
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
from ..specsettings import SpecSettings
from .wxplot import PlotCanvasPnl
from .netdfmondlgsbase import SpecPlotDlgBase, DfSpectrumPnl

mpl.use('WXAgg')


class DisplayData():
    """Container class for display data"""

    def __init__(self):
        """Container class for display data"""

        self.bearing: Optional[BearingData] = None
        self.y_values: Optional[list] = None
    

class RemoteSpectrumDlg(SpecPlotDlgBase):
    """Custom dialog class for data visualization."""

    def __init__(self, parent):
        """Dialog class for data visualization.

        :param parent: The parent window as wx.Window.
        """
        super().__init__(parent=parent)
        self.init_progress_bar(self.pgb_progress)
        self.init_dlg_buttons(self.sdbs_buttons, wx.OK)
        self.btn_ok.SetLabel("Close")
        self.SetTitle("Remote Bearing Spectrum Monitor")

        self.xmin = 390
        self.xmax = 395
        self.ymax = -20
        self.ymin = -120
        self.unit = LevelUnits.UNKNOWN
        self.spec_settings: Optional[SpecSettings] = None
        self.x_data_base = None
        self.y_data_base = None

        self._plots = []
        self._szr_plots = wx.WrapSizer(wx.HORIZONTAL, wx.WRAPSIZER_DEFAULT_FLAGS)
        self.scrw_plots.SetSizer(self._szr_plots)
        # self.Maximize(True)
        self.scrw_plots.Layout()
        self._szr_plots.Fit(self.scrw_plots)

    def set_xy_base_data(self, spec_settings: SpecSettings):
        self.spec_settings = spec_settings
        self.xmin = spec_settings.fstart / 1e6
        self.xmax = spec_settings.fstop / 1e6
        fstep = spec_settings.fstep / 1e6
        # self.x_data_base = np.arange(self.xmin, self.xmax, fsetp2)

        x_values = []
        xvalue = self.xmin
        for x in range(spec_settings.bin_cnt):
            x_values.append(xvalue)
            xvalue += fstep
        self.x_data_base = np.array(x_values)
        self.y_data_base = np.zeros(spec_settings.bin_cnt)
        self.ymax = spec_settings.ymax
        self.ymin = spec_settings.ymax - spec_settings.yrange
        self.unit = spec_settings.unit
        self.trace = spec_settings.trace_list[0]

    def create_plot(self, tag: str, title: str):
        plt = DfSpectrumPnl(self.scrw_plots)
        plt.tag = tag
        plt.plt_spectrum.axes.set_ylabel(str(self.spec_settings.unit))
        plt.plt_spectrum.axes.set_xlabel("Frequency/MHz")
        plt.plt_spectrum.axes.set_title(title + " " + str(self.trace))
        plt.plt_spectrum.axes.set_autoscale_on(False)
        plt.plt_spectrum.axes.set_autoscalex_on(False)
        plt.plt_spectrum.axes.set_autoscaley_on(False)
        plt.plt_spectrum.axes.grid(True)
        plt.plt_spectrum.axes.set_xbound(self.xmin, self.xmax)
        plt.plt_spectrum.axes.set_ybound(self.ymin, self.ymax)
        self._szr_plots.Add(plt, 1, wx.EXPAND | wx.ALL, 5)
        self._plots.append(plt)
        # plt.figure.tight_layout()
        plt.plt_spectrum.figure.subplots_adjust(left=0.2, right=0.9, bottom=0.2, top=0.9)

    def update_layout(self):
        self.scrw_plots.Layout()
        self._szr_plots.Fit(self.scrw_plots)
        self.Layout()
        self.Update()

    def update_progress(self, evt: ProgressEvent):
        """Receives data from thread and updates progressbar and message text

        :param evt: Event of type ProgressEvent.
        """
        # Handle progress
        self.update_progress_bar(evt, self.pgb_progress)

        # Handle message
        if evt.msg is not None:
            self.lbl_status.SetLabelText(evt.msg)

        # Handle data:
        if evt.data is not None and self.x_data_base is not None:
            for plt in self._plots:
                data = evt.data.get(plt.tag)
                if data is not None:
                    if data.bearing is not None:
                        plt.stxt_azimuth.SetLabel("{} °".format(data.bearing.azimuth))
                        plt.stxt_elevation.SetLabel("{} °".format(data.bearing.elevation))
                        plt.stxt_df_quality.SetLabel("{} %".format(data.bearing.df_quality))
                        plt.stxt_rms.SetLabel("{} {}".format(data.bearing.det_value, self.unit))
                    if data.y_values is not None:
                        plt.plt_spectrum.draw_data(self.x_data_base, data.y_values)

        # Handle buttons:
        if evt.btn_style:
            self.update_btn_style(evt.btn_style)

        # Handle icons:
        if evt.icon_style:
            # self.update_icon_style(self.bmp_icon, evt.icon_style)
            pass

        # Handle help text:
        if evt.help_text:
            self.help_text = evt.help_text

        self.Layout()
        self.Refresh()
        self.Update()


# This is just a python module containing helper classes/functions:
if __name__ == '__main__':
    pass
