#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import wx
import time
import datetime as dt
import threading
import concurrent.futures
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
from . import filehandling as fh


class DfNetLoc(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nslnetdf"
        self._scr_title = "DF and localization via network example V1.1"
        self._scr_description = "Takes bearings from several SignalShark units and calculates localization result."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 5
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        self.device_list = {} # Device list

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # load device list:
        self.device_list = fh.load_from_json(fh.DEVICE_LIST_NAME)

        # Check is device list exists:
        if not self.device_list:
            self.MessageBoxModal("Please run script \'Scan Devices\' to generate device list.", "No device list",
                                 wx.OK | wx.ICON_INFORMATION)
            return

        mthread = MeasDlgThread(main_gui=self.main_gui, callback=DfNetLoc._run_measurement,
                                args=[self.signalshark.addr, self.device_list])

        # Start the measurement thread and show a visualization (progress) dialog.
        dlg_result = mthread.start_measurement()
        if dlg_result == wx.ID_CANCEL:
            self.MessageBoxModal("Localization canceled", "Info")

        if mthread.result is not None:
            self.MessageBoxModal(str(mthread.result), "Localization - Result",
                          wx.OK | wx.ICON_INFORMATION)

        self.signalshark.disconnect()

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         master_ip: str, device_list):
        """Main method of the measurement thread that handles a time consuming measurement.

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param master_ip: Custom parameter to handover the ip address of the SignalShark
        :param devices_ip_list: Custom parameter to handover the device list
        :return:
        """
        if not device_list:
            return wait_for_stopevent("Empty device list")

        # Try to connect to master:
        signalshark = SignalSharkDev(master_ip)
        try:
            if not signalshark.connect():
                return wait_for_stopevent("Cannot connect to master device!")

            # Check if master is ready for bearing:
            task_type, task_name = signalshark.scpi.task.get_selected()
            if task_type != TaskTypes.AUTO_DF:
                signalshark.disconnect()
                return wait_for_stopevent("Please add Auto DF task to master and configure it!\r\nAfterwards run sync script.")

            ftune = signalshark.scpi.bearing.get_frequency_tune()
            signalshark.disconnect()
            time.sleep(0.1)

            # get a bearing from every device:
            # ----------------------------------------------------------------------------------
            if device_list is None or len(device_list) < 1:
                return wait_for_stopevent("No device available!")

            bearings = []
            with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
                future_list = []

                # Submit ip addresses for bearing:
                for key in device_list:
                    future = executor.submit(DfNetLoc.get_bearing, key)
                    future_list.append(future)

                # Get response:
                stop_cnt = 0
                for future in concurrent.futures.as_completed(future_list):
                    try:
                        bearing = future.result()
                        if bearing is not None:
                            bearings.append(bearing)
                    except:
                        pass

            # Save bearings and calculate localization
            # ----------------------------------------------------------------------------------
            if len(bearings) < 1:
                wait_for_stopevent("No bearings available!")

            # Connect to master:
            signalshark.connect()
            if not signalshark.connect():
                return wait_for_stopevent("Cannot connect to master device!")

            # Setup directory:
            old_working_dir = Path(signalshark.scpi.dlogger.get_dloger_working_directory())
            if old_working_dir.stem == "Datalogger":
                base_working_dir = old_working_dir
            else:
                base_working_dir = old_working_dir.parent

            new_folder_name = dt.datetime.now().strftime("ScrBearing_%Y-%m-%d_%H-%M")
            new_working_dir = str(base_working_dir.joinpath(new_folder_name))
            real_path = signalshark.scpi.dlogger.get_directory_add(new_working_dir)
            signalshark.scpi.dlogger.set_dloger_working_directory(real_path)

            # Save bearings:
            for bearing in bearings:
                signalshark.scpi.dlogger.save_external_bearing(ftune, bearing, "", bearing.comment,
                                                               bearing.comment)
            # Init master device for localization:
            signalshark.scpi.check_add_view(ViewTypes.MAP)
            signalshark.scpi.map.set_mode(MapModes.DISCRETE_LOCALIZATION)
            signalshark.scpi.map.set_localization_bearing_error(5)
            signalshark.scpi.map.set_localization_los_probability(10)
            signalshark.scpi.map.set_localization_min_speed(1)
            signalshark.scpi.map.set_localization_min_speed_enable(False)
            signalshark.scpi.map.set_localization_min_dfquality(50)
            signalshark.scpi.map.set_localization_min_dfquality_enable(True)

            # Calculate localization:
            update_progress(msg="Calculating localization", btn_style=wx.ID_NONE)
            signalshark.scpi.map.localization_start()
            time.sleep(4)
            progr = 0.0
            while progr < 1.0:
                data = signalshark.scpi.system.get_command_progress()
                if data:
                    progr = data.get_percent_done()
                    update_progress(progress=progr)
                time.sleep(1)

            signalshark.scpi.map.localization_stop()
            signalshark.scpi.check_no_error()
            loc_data = signalshark.scpi.map.get_localization_data()

            signalshark.disconnect()

            # Build return message:
            if loc_data is None:
                msg = "No valid localization available"
            else:
                msg = "Transmitter longitude: {}{}".format(loc_data.coordinate.latitude, CRLF) + \
                      "Transmitter latitude:  {}{}".format(loc_data.coordinate.longitude, CRLF)

            return msg
        except Exception as e:
            # Do some error handling
            wait_for_stopevent(str(e))
            return "Unexpected error while localization!"
        finally:
            if signalshark is not None:
                signalshark.disconnect()

    @staticmethod
    def get_bearing(device_ip: str) -> Optional[BearingData]:
        """Requests bearing data from the given SignalShark device.

        :param device_ip: The IP address of the SignalShark.
        :type device_ip: str
        :return: Returns the aquired beraing data or None.
        :rtype: BearingData
        """
        signalshark = SignalSharkDev(device_ip)
        retvalue = None
        try:
            if signalshark.connect():
                device_pn = signalshark.scpi.device.get_part_number()
                device_sn = signalshark.scpi.device.get_serial_number()
                dev_id = "{}_{}".format(device_pn, device_sn)
                scan_number = 0
                for x in range(3):
                    scan_number = signalshark.scpi.bearing.get_data_update(scan_number)
                    if scan_number > 0:
                        retvalue = signalshark.scpi.bearing.get_data_all()
                        if retvalue is not None:
                            retvalue.comment = "{} {}".format(dev_id, device_ip)
                            break
                signalshark.disconnect()
            return retvalue
        except:
            return None
