#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################

import wx
import threading
from typing import List, Optional
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
from .gui.netdfmondlgs import RemoteSpectrumDlg, DisplayData
from .specsettings import SpecSettings
from . import filehandling as fh


class DfNetSpecMon(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nslnetdf"
        self._scr_title = "Remote DF Spectrum Monitoring V1.1"
        self._scr_description = "Displays spectrum data from all DF receivers in the network."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 4
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        self.devices_ip_list = []

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # load device list:
        device_list = fh.load_from_json(fh.DEVICE_LIST_NAME)

        # Check is device list exists:
        if not device_list:
            self.MessageBoxModal("Please run script \'Scan Devices\' to generate device list.", "No device list",
                                 wx.OK | wx.ICON_INFORMATION)
            return

        # Query spectrum plot settings from master device:
        self.signalshark.connect()
        if self.signalshark.scpi.check_error():
            self.MessageBoxModal("Can't connect to device.", "Connection error",
                                 wx.OK | wx.ICON_INFORMATION)
            return

        # Check if master is ready for bearing:
        # task_type, task_name = self.signalshark.scpi.task.get_selected()
        # if task_type != TaskTypes.AUTO_DF:
        #     self.MessageBoxModal("Please add an \'Auto DF\' Task.", "Settings error",
        #                          wx.OK | wx.ICON_INFORMATION)
        #     return

        # Ask user to select a trace
        trace_list = []
        for item in self.signalshark.scpi.spectrum.get_trace_list():
            trace_list.append(str(item))

        selected_trace_str = self.config.get("selected_trace", "")
        selected_trace = LevDetectorModes.from_string(selected_trace_str)
        with tb.ListMsgBox(self.frm_dlghelper, "Trace: ", trace_list, "", str(selected_trace), "Select a trace") as dlg:
            if self.ShowDlgModalTop(dlg) == wx.ID_OK:
                selected_trace = LevDetectorModes.from_string(dlg.value)
        if selected_trace == LevDetectorModes.UNKNOWN:
            self.MessageBoxModal("Trace selection is mandatory!", "Trace Selection", wx.OK | wx.ICON_INFORMATION)
            return
        self.config["selected_trace"] = str(selected_trace)

        # Get spectrum settings
        spec_settings = SpecSettings()
        spec_settings.get_from_device(self.signalshark)
        spec_settings.trace_list.clear()
        spec_settings.trace_list.append(selected_trace)

        # Add spectrum display dialog.
        spec_mon_dlg = RemoteSpectrumDlg(self.main_gui)
        spec_mon_dlg.set_xy_base_data(spec_settings)

        master_pn = self.signalshark.scpi.device.get_part_number()
        master_sn = self.signalshark.scpi.device.get_serial_number()
        master_id = "{}_{}".format(master_pn, master_sn)
        spec_mon_dlg.create_plot(master_id, "Master {} {}".format(master_id, self.signalshark.addr))

        for key in device_list:
            device_info = device_list[key]
            pn_number = device_info["PN"]
            sn_number = device_info["SN"]
            dev_id = "{}_{}".format(pn_number, sn_number)
            if dev_id != master_id:
                spec_mon_dlg.create_plot(dev_id, dev_id + " " + key)
                self.devices_ip_list.append(key)
        spec_mon_dlg.update_layout()
        self.signalshark.disconnect()

        #  Run measurement in additional task to keep GUI alive.
        mthread = MeasDlgThread(main_gui=self.main_gui, dlg=spec_mon_dlg,
                                callback=DfNetSpecMon._run_measurement,
                                args=[self.signalshark.addr, self.devices_ip_list, selected_trace])
        mthread.start_measurement()

        self.signalshark.disconnect()

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         master_ip: str, devices_ip_list, selected_trace: LevDetectorModes):
        """Main method of the measurement thread that handles a time consuming measurement.

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param master_ip: Custom parameter to handover the ip address of the SignalShark
        :param devices_ip_list: Custom parameter to handover the device list
        :param selected_trace: The selected trace to display
        :return:
        """
        master = SignalSharkDev(master_ip)
        devices = []
        try:
            if master.connect():
                devices.append(master)
                for ip in devices_ip_list:
                    device = SignalSharkDev(ip)
                    if device.connect():
                        devices.append(device)
                        device.scpi.spectrum.set_trace_enable(selected_trace, True)
            else:
                return wait_for_stopevent("Cannot connect to master device!")

            # Get spectrum from every device:
            while not stopevent.is_set():
                try:
                    data = {}
                    for device in devices:
                        dev_pn = device.scpi.device.get_part_number()
                        dev_sn = device.scpi.device.get_serial_number()
                        dev_id = "{}_{}".format(dev_pn, dev_sn)
                        scan_cnt = device.scpi.spectrum.get_data_update()
                        if scan_cnt > 0:
                            plt_data = DisplayData()
                            plt_data.bearing = device.scpi.bearing.get_data_all()
                            plt_data.y_values = device.scpi.spectrum.get_data_level(selected_trace)
                            data[dev_id] = plt_data
                    update_progress(data=data)
                except ValueError:
                    break
                stopevent.wait(0.5)
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e))
        finally:
            if devices is not None:
                for device in devices:
                    device.disconnect()
            if master is not None:
                master.disconnect()


# This is just a python module containing helper classes/functions:
if __name__ == '__main__':
    pass
