#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from typing import List, Callable, Optional, Any
import time
import threading
from concurrent.futures import ThreadPoolExecutor, as_completed
from queue import Queue
import socket
import ifaddr
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *


def get_nic_list() -> list:
    """Gets a list of all available network interface cards (NICs).

    :return: Returns a list of available NICs or an empty list.
    :rtype: list
    """
    try:
        nic_list = []
        adapters = ifaddr.get_adapters()
        for adapter in adapters:
            for addr in adapter.ips:
                if addr.is_IPv4:
                    if not addr.ip.startswith("169.254."):
                        ip = str(addr.ip)
                        name = str(addr.nice_name)
                        nic_list.append("{}; {}".format(ip, name))
        return nic_list
    except:
        return []


def ip_request(scan_ip: str, request_callback: Optional[Callable[[str], Any]] = None) -> Any:
    """Method to execute an IP request.

    :param scan_ip: Actual IP to scan.
    :type scan_ip: str
    :param request_callback: An optional request method, that is used to generate a queue entry.
    If None, the IP address of the detected device will be used as result in the queue.
    :type request_callback: Callable
    :return: Request result
    :rtype: Any
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, True)
    try:
        result = s.connect_ex((scan_ip, 5300))
        s.close()

        if result == 0:
            if request_callback:
                return request_callback(scan_ip)
            else:
                return scan_ip
    except:
        return None


def scan_network(nic_ip: str, max_workers: int = 4, timeout: float = 3.0,
                 request_callback: Optional[Callable[[str], Any]] = None) -> list:
    """Scans a network for all available devices.

    :param nic_ip: The IP of the network interface card (NIC)
    :param max_workers: Maximum number of threads.
    :param timeout: Execution timeout time.
    :param request_callback: An optional request method, that is used to generate a queue entry.
    If None, the IP address of the detected device will be used as result in the queue.
    :return: Returns a list of results.
    """
    result_list = []
    try:
        nic_ip_lst = nic_ip.split(".")
        if len(nic_ip_lst) != 4:
            return []

        base_ip = nic_ip_lst[0] + "." + nic_ip_lst[1] + "." + nic_ip_lst[2] + "."
        # We can use a with statement to ensure threads are cleaned up promptly
        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            future_list = []
            for i in range(1, 254):
                ip = base_ip + str(i)
                future = executor.submit(ip_request, ip, request_callback)
                future_list.append(future)

            for future in as_completed(future_list, timeout=timeout):
                try:
                    data = future.result()
                    result_list.append(data)
                except:
                    pass
        return result_list
    except:
        return []


# This is just a python module containing helper classes/functions:
if __name__ == '__main__':
    pass



