#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import socket
import time
import threading
import concurrent.futures
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
from . import nettools as nt
from .gui.netdfdlgs import DeviceListDlg


class ScanNetDevices(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nslnetdf"
        self._scr_title = "Network devices scan V1.1"
        self._scr_description = "Scans the network for signalshark devices and saves the result as a json dictionary"
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 1
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        # self.my_variable = None

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # scan available NICs:
        nic_list = nt.get_nic_list()
        last_nic = self.config.get("last_nic", "")

        # Select a NIC IP:
        selected_nic = None
        with tb.ListMsgBox(self.frm_dlghelper, "Net: ", nic_list, "", last_nic, "Select network adapter") as dlg:
            if self.ShowDlgModalTop(dlg) == wx.ID_OK:
                selected_nic = dlg.value
        if not selected_nic:
            self.MessageBoxModal("A NIC must be selected!", "NIC Selection", wx.OK | wx.ICON_INFORMATION)
            return
        nic_ip = selected_nic.split(";")[0]
        self.config["last_nic"] = selected_nic

        # Setup device table dialog:
        dev_tbl_dlg = DeviceListDlg(self.main_gui)
        dev_tbl_dlg.update_btn_style(wx.CANCEL | wx.HELP)
        dev_tbl_dlg.help_text = """\
        Network devices scan:
        This tool scans your network for connected SignalShark devices.
        You can change some basic settings and add a text note.
        
        If you press the OK button, all devices that are marked by the
        check box will be saved to the file 'devicelist.json'
        and they will be used for localization.
         \
        """

        # Add measurement thread to do asynchronious network scan (to keep GUI alive)
        mthread = MeasDlgThread(main_gui=self.main_gui, dlg=dev_tbl_dlg,
                                callback=ScanNetDevices._run_measurement, args=[nic_ip])
        mthread.start_measurement()

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         nic_ip: str):
        """ Main method of the measurement thread that handles a time consuming measurement.

        Please do NOT use any wxPython elements like a MessageBox or a Dialog here!
        They are only permitted in the GUI thread ('_run_script' method).
        Please use the "update_progress" method to display information!

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param nic_ip: Custom parameter to handover the nic_ip
        :return:
        """
        default_timeout = 3
        try:
            # Prepare progress
            step = 1.0/253
            progr = 0

            # Check IP
            nic_ip_lst = nic_ip.split(".")
            if len(nic_ip_lst) != 4:
                return False

            # Decreas socket timeout time:
            default_timeout = socket.getdefaulttimeout()
            socket.setdefaulttimeout(0.5)

            # Start network scan utilizing 10 worker threads:
            update_progress(msg="Scanning network...")
            base_ip = nic_ip_lst[0] + "." + nic_ip_lst[1] + "." + nic_ip_lst[2] + "."
            with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
                future_list = []

                # Submit ip addresses to scan:
                for i in range(1, 254):
                    scan_ip = base_ip + str(i)
                    future = executor.submit(ScanNetDevices.ip_request, scan_ip)
                    future_list.append(future)

                # Get response:
                cnt_val = 0
                stop_cnt = 0
                for future in concurrent.futures.as_completed(future_list):
                    cnt_val += 1
                    progr = cnt_val * step
                    try:
                        data = future.result()
                        if data is not None:
                            # Ensure, that progress with data is updated!
                            update_progress(data=data, progress=progr, force_update=True)
                        else:
                            update_progress(progress=progr)
                    except:
                        pass

                    # Cancel undone ip requests if stopevent is set:
                    stop_cnt += 1
                    if stop_cnt > 50:
                        stop_cnt = 0
                        stopevent.wait(0.1)
                        if stopevent.is_set():
                            done, not_done = concurrent.futures.wait(future_list, timeout=0)
                            if not_done:
                                for future_not_done in not_done:
                                    future_not_done.cancel()

            wait_for_stopevent(msg="Scan done", btn_style=(wx.CANCEL | wx.OK | wx.HELP))
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e))
        finally:
            socket.setdefaulttimeout(default_timeout)

    @staticmethod
    def ip_request(scan_ip: str) -> Optional[dict]:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, True)
        sigshark = None
        retvalue = None
        try:
            result = s.connect_ex((scan_ip, 5300))
            s.close()
            s = None
            if result == 0:
                sigshark = SignalSharkDev(scan_ip, 5300, 3)
                if sigshark.connect():
                    part_number = sigshark.scpi.device.get_part_number()
                    serial_num =  sigshark.scpi.device.get_serial_number()
                    retvalue = {"IP": sigshark.addr, "MAC": sigshark.device_mac, "PN": part_number, "SN": serial_num,
                                "IDN": sigshark.device_info}
                sigshark.disconnect()
            return retvalue
        except:
            if s:
                s.close()
            if sigshark:
                sigshark.disconnect()
            return None